--******************************************************************************************************************************
--* Licensed Materials - Property of IBM                                                                                       *
--* (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.                                                                    *
--*                                                                                                                            *
--*US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp*
--******************************************************************************************************************************



USE $(DBNAME)
GO

CREATE SCHEMA AUTHORIZATION $(CURRENT_SCHEMA)
GO
set ANSI_NULLS ON
set QUOTED_IDENTIFIER ON
GO
CREATE FUNCTION $(CURRENT_SCHEMA).REPLACE_SCHEMA (@STATEMENT VARCHAR(4000), @ODS_SCHEMA VARCHAR(20),@DM_SCHEMA VARCHAR(20),@REP_USER VARCHAR(20))
RETURNS NVARCHAR(4000)
AS
BEGIN
		DECLARE @v_text 										VARCHAR(4000);
		DECLARE @v_ods_schema 							VARCHAR(20);
		DECLARE @v_dm_schema 							VARCHAR(20);
		DECLARE @v_rep_user								VARCHAR(20);
		
		SET @v_text=@STATEMENT;
		SET @v_ods_schema=@ODS_SCHEMA + '.';
		SET @v_dm_schema=@DM_SCHEMA + '.';
		SET @v_rep_user=@REP_USER;
		
		SET @v_text=COALESCE(REPLACE(@v_text,'RIDW.',@v_dm_schema),@v_text);
		SET @v_text=COALESCE(REPLACE(@v_text,'RIODS.',@v_ods_schema),@v_text);
		SET @v_text=COALESCE(REPLACE(@v_text,'REPUSER',@v_rep_user),@v_text);
		RETURN @v_text;
END
GO


CREATE PROCEDURE $(CURRENT_SCHEMA).PROCESS_IMPLICIT_CONSTR(@SCHEMA_NAME VARCHAR(20), @TABLE_NAME VARCHAR(100), @COLUMN_NAME VARCHAR(100), @CONSTR_TYPE VARCHAR(20))
AS    
BEGIN    
			DECLARE @v_sqlcode INT; 
            SET @v_sqlcode=0;  	
			DECLARE @v_constr_defval VARCHAR(100) ;		

			BEGIN TRANSACTION Timplicit
			IF (@CONSTR_TYPE = 'D')
					SELECT @v_constr_defval=d.name
			    FROM sys.default_constraints AS d
				JOIN sys.objects AS o ON o.object_id = d.parent_object_id AND o.name=@TABLE_NAME
				JOIN sys.columns AS c ON c.object_id = o.object_id AND c.column_id = d.parent_column_id AND c.name=@COLUMN_NAME
				JOIN sys.schemas AS s ON s.schema_id = o.schema_id AND s.name=@SCHEMA_NAME;
				
			IF (@v_constr_defval IS NOT NULL) 
						EXECUTE ('ALTER TABLE ' + @SCHEMA_NAME + '.' + @TABLE_NAME + ' DROP CONSTRAINT ' + @v_constr_defval);
			COMMIT TRANSACTION Timplicit;
END
GO

CREATE PROCEDURE $(CURRENT_SCHEMA).PROCESS_ALL_CONSTR(@SCHEMA_NAME VARCHAR(20), @TABLE_NAME VARCHAR(100), @COLUMN_NAME VARCHAR(100))
AS    
BEGIN    
			DECLARE @v_sqlcode INT; 
      		SET @v_sqlcode=0;  	
			DECLARE @v_constr_defval VARCHAR(100) ;		

			BEGIN TRANSACTION TallConstr		
			DECLARE implicitCur CURSOR FOR 
 				SELECT c.CONSTRAINT_NAME
				FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE AS c
				WHERE c.TABLE_SCHEMA=@SCHEMA_NAME AND c.TABLE_NAME=@TABLE_NAME and c.COLUMN_NAME=@COLUMN_NAME
				UNION
			    SELECT d.name
			    FROM sys.default_constraints AS d
				JOIN sys.objects AS o ON o.object_id = d.parent_object_id AND o.name=@TABLE_NAME
				JOIN sys.columns AS c ON c.object_id = o.object_id AND c.column_id = d.parent_column_id AND c.name=@COLUMN_NAME
				JOIN sys.schemas AS s ON s.schema_id = o.schema_id AND s.name=@SCHEMA_NAME;
  			OPEN implicitCur;
  			FETCH NEXT FROM implicitCur INTO @v_constr_defval;
  			WHILE @@FETCH_STATUS = 0
  			BEGIN
					IF (@v_constr_defval IS NOT NULL) 
								EXECUTE ('ALTER TABLE ' + @SCHEMA_NAME + '.' + @TABLE_NAME + ' DROP CONSTRAINT ' + @v_constr_defval);	
					FETCH NEXT FROM implicitCur INTO @v_constr_defval;
  	  		END  
  			CLOSE implicitCur;
  			DEALLOCATE implicitCur;
			COMMIT TRANSACTION TallConstr;
END
GO




CREATE PROCEDURE $(CURRENT_SCHEMA).GET_REFERENCE_DEFINE(@SCHEMA_NAME VARCHAR(20), @TABLE_NAME VARCHAR(100), @COLUMN_NAME VARCHAR(100), @TMPREFTAB VARCHAR(100))
AS    
BEGIN    
		DECLARE @v_sqlcode INT; 
		SET @v_sqlcode=0;
		DECLARE @v_refdefine VARCHAR(2048);
		DECLARE @v_constname VARCHAR(100);
		DECLARE @v_schemaname  VARCHAR(100);
		DECLARE @v_tabname  VARCHAR(100);
		DECLARE @v_colname  VARCHAR(100);
		DECLARE @v_deleteaction INT;
		DECLARE @v_updateaction INT;
		DECLARE @v_actstr VARCHAR(2048);
		
		DECLARE refdefCur CURSOR FOR  					
			SELECT object_name(f.constid), s.name, object_name(f.fkeyid), c.name, fk.delete_referential_action, fk.update_referential_action
					FROM sys.sysforeignkeys AS f
							JOIN sys.objects AS o ON f.fkeyid = o.object_id
							JOIN sys.schemas AS s ON s.schema_id = o.schema_id 
							JOIN sys.columns AS c ON c.object_id = o.object_id AND c.column_id = f.fkey
							JOIN sys.foreign_keys AS fk ON fk.object_id = f.constid AND fk.schema_id = s.schema_id
							JOIN sys.objects AS o2 ON f.rkeyid = o2.object_id AND o2.name=@TABLE_NAME
							JOIN sys.schemas AS s2 ON s2.schema_id = o2.schema_id AND s2.name=@SCHEMA_NAME
							JOIN sys.columns AS c2 ON c2.object_id = o2.object_id AND c2.column_id = f.rkey AND c2.name=@COLUMN_NAME;

		OPEN refdefCur;
		FETCH NEXT FROM refdefCur INTO @v_constname, @v_schemaname, @v_tabname, @v_colname, @v_deleteaction, @v_updateaction;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				SET @v_actstr = '';
				IF (@v_updateaction = 1) 
						SET @v_actstr = @v_actstr + ' ON UPDATE CASCADE ';
				IF (@v_deleteaction = 1) 
						SET @v_actstr = @v_actstr + ' ON DELETE CASCADE ';
				SET @v_refdefine='ALTER TABLE '+ @v_schemaname+'.'+@v_tabname+' ADD CONSTRAINT '+ @v_constname +' FOREIGN KEY('+@v_colname+') REFERENCES '+@SCHEMA_NAME+'.'+@TABLE_NAME+'('+@COLUMN_NAME+')'+@v_actstr;
				EXECUTE ('INSERT ' + @TMPREFTAB + ' VALUES(''' + @v_schemaname + ''','''+@v_constname + ''','''+ @v_tabname + ''',''' + @v_refdefine +''')');	
				FETCH NEXT FROM refdefCur INTO @v_constname, @v_schemaname, @v_tabname, @v_colname, @v_deleteaction, @v_updateaction;
		END  
		CLOSE refdefCur;
		DEALLOCATE refdefCur;
				
END
GO


CREATE FUNCTION $(CURRENT_SCHEMA).GET_IDXCOL_LIST (@OBJECT_ID INT, @INDEX_ID INT)
RETURNS NVARCHAR(2048)
AS
BEGIN
	DECLARE	@v_colname			VARCHAR(100);
	DECLARE	@v_colseq			INT;
	DECLARE	@v_maxcolseq		INT;
	DECLARE @v_sqlcode INT; 
	SET @v_sqlcode=0;
	DECLARE	@v_text				VARCHAR(100);

	SET @v_text='';
	DECLARE idxcolCur CURSOR FOR
		SELECT (SELECT c.name FROM sys.columns c WHERE object_id = @OBJECT_ID AND column_id = idxcol.column_id) AS COLNAME, index_column_id, 
				(SELECT MAX(index_column_id) FROM sys.index_columns WHERE object_id = @OBJECT_ID AND index_id = @INDEX_ID) AS MAXCOLSEQ
				FROM sys.index_columns idxcol 
					WHERE object_id = @OBJECT_ID AND index_id = @INDEX_ID
				ORDER BY index_column_id;

	OPEN idxcolCur;
	FETCH NEXT FROM idxcolCur INTO @v_colname, @v_colseq, @v_maxcolseq;
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @v_text=@v_text+@v_colname;
		IF (@v_colseq <> @v_maxcolseq) 
				SET @v_text=@v_text+',';
			
		FETCH NEXT FROM idxcolCur INTO @v_colname, @v_colseq, @v_maxcolseq;	
	END
	CLOSE idxcolCur;
	DEALLOCATE idxcolCur;
	RETURN @v_text;
END
GO


CREATE PROCEDURE $(CURRENT_SCHEMA).GET_INDEX_DEFINE(@SCHEMA_NAME VARCHAR(20), @TABLE_NAME VARCHAR(100), @COLUMN_NAME VARCHAR(100), @TMPIDXTAB VARCHAR(100))
AS    
BEGIN    
		DECLARE @v_sqlcode INT; 
		SET @v_sqlcode=0;
		DECLARE @v_idxdefine VARCHAR(2048);
		DECLARE @v_idxname VARCHAR(100);
		DECLARE @v_schemaname  VARCHAR(100);
		DECLARE @v_tabname  VARCHAR(100);
		DECLARE @v_idxcol_list  VARCHAR(1024);
		DECLARE @v_datspace VARCHAR(100);
		DECLARE @v_object_id INT;
		DECLARE @v_index_id INT;
		
		
		DECLARE idxdefCur CURSOR FOR  					
			SELECT i.object_id, i.index_id, i.name, (SELECT name FROM sys.data_spaces WHERE data_space_id=i.data_space_id)
					FROM sys.indexes AS i
							JOIN sys.objects AS o ON i.object_id = o.object_id AND o.name=@TABLE_NAME
							JOIN sys.schemas AS s ON s.schema_id = o.schema_id AND s.name=@SCHEMA_NAME
					WHERE i.is_primary_key != 1 AND EXISTS (SELECT c.object_id FROM sys.index_columns idxcol
										JOIN sys.columns AS c ON c.object_id = o.object_id AND c.object_id = idxcol.object_id AND i.index_id = idxcol.index_id 
										AND c.column_id = idxcol.column_id AND c.name=@COLUMN_NAME)												
					;

		OPEN idxdefCur;
		FETCH NEXT FROM idxdefCur INTO @v_object_id, @v_index_id, @v_idxname, @v_datspace;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				SET @v_idxcol_list=$(CURRENT_SCHEMA).GET_IDXCOL_LIST (@v_object_id, @v_index_id);
				SET @v_idxdefine='CREATE INDEX '+@v_idxname+' ON ' + @SCHEMA_NAME + '.' + @TABLE_NAME + '(' + @v_idxcol_list + ') ON ' + @v_datspace;
				EXECUTE ('INSERT ' + @TMPIDXTAB + ' VALUES(''' + @SCHEMA_NAME +''','''+ @v_idxname +''',''' + @TABLE_NAME + ''',''' + @v_idxdefine + ''')');
				FETCH NEXT FROM idxdefCur INTO @v_object_id, @v_index_id, @v_idxname, @v_datspace;
		END
		CLOSE idxdefCur;
		DEALLOCATE idxdefCur; 
				
END
GO


CREATE PROCEDURE $(CURRENT_SCHEMA).GET_DEFAULT_CONSTR(@SCHEMA_NAME VARCHAR(20), @TABLE_NAME VARCHAR(100), @COLUMN_NAME VARCHAR(100), @TMPDEFTAB VARCHAR(100))
AS    
BEGIN    
		DECLARE @v_sqlcode INT; 
    SET @v_sqlcode=0;  	
		DECLARE @v_constr_defval VARCHAR(100) ;	
		DECLARE @v_definition VARCHAR(100) ;	
		DECLARE @v_defdefine VARCHAR(2048) ;

		DECLARE DCCur CURSOR FOR 
					SELECT d.name, CASE WHEN c.system_type_id =61 THEN SUBSTRING(d.definition, 2, len(d.definition)-2)
															WHEN c.system_type_id in (231,239) THEN RTRIM(REPLACE(SUBSTRING(d.definition, 2, len(d.definition)-2) ,CHAR(39), CHAR(39)+CHAR(39)))
															ELSE SUBSTRING(d.definition, 3, len(d.definition)-4) END AS definition
			    FROM sys.default_constraints AS d
				JOIN sys.objects AS o ON o.object_id = d.parent_object_id AND o.name=@TABLE_NAME
				JOIN sys.columns AS c ON c.object_id = o.object_id AND c.column_id = d.parent_column_id AND c.name=@COLUMN_NAME
				JOIN sys.schemas AS s ON s.schema_id = o.schema_id AND s.name=@SCHEMA_NAME;
			
		OPEN DCCur;
		FETCH NEXT FROM DCCur INTO @v_constr_defval, @v_definition;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				SET @v_defdefine='ALTER TABLE ' + @SCHEMA_NAME + '.' + @TABLE_NAME + ' ADD CONSTRAINT ' + @v_constr_defval + ' DEFAULT ' + @v_definition + ' FOR ' + @COLUMN_NAME;
				EXECUTE ('INSERT ' + @TMPDEFTAB + ' VALUES(''' + @SCHEMA_NAME + ''',''' + @v_constr_defval + ''',''' + @TABLE_NAME + ''',''' + @v_defdefine + ''')');		
				FETCH NEXT FROM DCCur INTO @v_constr_defval, @v_definition;
		END
		CLOSE DCCur;
		DEALLOCATE DCCur;
END
GO


CREATE PROCEDURE $(CURRENT_SCHEMA).PROCESS_COLUMN_UPDATE(@SCHEMA_NAME VARCHAR(20), @TABLE_NAME VARCHAR(100), @COLUMN_NAME VARCHAR(100), @COLOMN_UPDATE_DDL VARCHAR(100))
AS    
BEGIN    
		DECLARE @v_sqlcode INT; 
		SET @v_sqlcode=0;
		DECLARE @v_sqlstate CHAR(5) ;
		SET @v_sqlstate= '00000';
		
		DECLARE @v_defdefine VARCHAR(2048);
		DECLARE @v_refdefine VARCHAR(2048);
		DECLARE @v_idxdefine VARCHAR(2048);
		DECLARE @v_schemaname  VARCHAR(100);
		DECLARE @v_tabname  VARCHAR(100);
		DECLARE @v_constname VARCHAR(100);
		DECLARE @v_idxname VARCHAR(100);
		DECLARE @v_idxcol_list  VARCHAR(1024);
		DECLARE @v_datspace VARCHAR(100);


		-- DEFINE A CACHE FOR DEFAULT VAULE ON THE COLUMN_NAME
		-- DECLARE @vt_defdefine table(constschema VARCHAR(20), constname VARCHAR(32), tabname VARCHAR(100), defdefine VARCHAR(2048));
		IF object_id('tempdb..#vt_defdefine') IS NOT NULL
				DROP TABLE #vt_defdefine;
		CREATE TABLE #vt_defdefine (constschema VARCHAR(20), constname VARCHAR(32), tabname VARCHAR(100), defdefine VARCHAR(2048));
				
		-- DEFINE A CACHE FOR REFERENCES ON THE COLUMN_NAME
		-- DECLARE @vt_refdefine table(constschema VARCHAR(20), constname VARCHAR(32), tabname VARCHAR(100), refdefine VARCHAR(2048));
		IF object_id('tempdb..#vt_refdefine') IS NOT NULL
				DROP TABLE #vt_refdefine;
		CREATE TABLE #vt_refdefine (constschema VARCHAR(20), constname VARCHAR(32), tabname VARCHAR(100), refdefine VARCHAR(2048));
		
		-- DEFINE A CACHE FOR INDEXES ON THE COLUMN_NAME
		-- DECLARE @vt_idxdefine table(indexschema VARCHAR(20), indexname VARCHAR(32), tabname VARCHAR(100), idxdefine VARCHAR(2048));
		IF object_id('tempdb..#vt_idxdefine') IS NOT NULL
				DROP TABLE #vt_idxdefine;
		CREATE TABLE #vt_idxdefine (indexschema VARCHAR(20), indexname VARCHAR(32), tabname VARCHAR(100), idxdefine VARCHAR(2048));


		-- GET DEFAULT CONSTRAINT DEFINE
		EXEC $(CURRENT_SCHEMA).GET_DEFAULT_CONSTR @SCHEMA_NAME, @TABLE_NAME, @COLUMN_NAME, '#vt_defdefine';
		
		-- GET REFERENCES DEFINE
		EXEC $(CURRENT_SCHEMA).GET_REFERENCE_DEFINE @SCHEMA_NAME, @TABLE_NAME, @COLUMN_NAME, '#vt_refdefine';
		
		-- GET INDEXES DEFINE
		EXEC $(CURRENT_SCHEMA).GET_INDEX_DEFINE @SCHEMA_NAME, @TABLE_NAME, @COLUMN_NAME, '#vt_idxdefine';


		-- DROP CONSTRAINTS ON COLUMN
		DECLARE defCur CURSOR FOR SELECT constschema, constname, tabname, defdefine FROM #vt_defdefine; 
		OPEN defCur;
		FETCH NEXT FROM defCur INTO @v_schemaname, @v_constname, @v_tabname, @v_defdefine;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				EXECUTE ('ALTER TABLE ' + @v_schemaname + '.' + @v_tabname + ' DROP CONSTRAINT ' + @v_constname);
				PRINT ('ALTER TABLE ' + @v_schemaname + '.' + @v_tabname + ' DROP CONSTRAINT ' + @v_constname);
				FETCH NEXT FROM defCur INTO @v_schemaname, @v_constname, @v_tabname, @v_defdefine;
		END  
		CLOSE defCur;
		DEALLOCATE defCur;

		-- DROP CONSTRAINTS ON COLUMN
		DECLARE refCur CURSOR FOR SELECT constschema, constname, tabname, refdefine FROM #vt_refdefine; 
		OPEN refCur;
		FETCH NEXT FROM refCur INTO @v_schemaname, @v_constname, @v_tabname, @v_refdefine;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				EXECUTE ('ALTER TABLE ' + @v_schemaname + '.' + @v_tabname + ' DROP CONSTRAINT ' + @v_constname);
				PRINT ('ALTER TABLE ' + @v_schemaname + '.' + @v_tabname + ' DROP CONSTRAINT ' + @v_constname);
				FETCH NEXT FROM refCur INTO @v_schemaname, @v_constname, @v_tabname, @v_refdefine;
		END  
		CLOSE refCur;
		DEALLOCATE refCur;

		-- DROP INDEXES ON COLUMN
		DECLARE idxCur CURSOR FOR SELECT indexschema, indexname, tabname, idxdefine FROM #vt_idxdefine; 
		OPEN idxCur;
		FETCH NEXT FROM idxCur INTO @v_schemaname, @v_idxname, @v_tabname, @v_idxdefine;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				EXECUTE ('DROP INDEX ' + @v_idxname  + ' ON ' + @v_schemaname + '.' + @v_tabname);
				PRINT ('DROP INDEX ' + @v_idxname  + ' ON ' + @v_schemaname + '.' + @v_tabname);
				FETCH NEXT FROM idxCur INTO @v_schemaname, @v_idxname, @v_tabname, @v_idxdefine;
		END
		CLOSE idxCur;
		DEALLOCATE idxCur;
		
		-- EXECUTE UPDATE MIGRATION DDL ON THE COLUMN
		EXECUTE (@COLOMN_UPDATE_DDL);
		SELECT @v_sqlstate=@@ERROR;
		IF (@v_sqlstate <> 0)
					RETURN @v_sqlstate;

		-- REINSTATE DEFAULT CONSTRAINTSON COLUMN
		DECLARE defCur CURSOR FOR SELECT defdefine FROM #vt_defdefine; 
		OPEN defCur;
		FETCH NEXT FROM defCur INTO @v_defdefine;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				EXECUTE (@v_defdefine);
				FETCH NEXT FROM defCur INTO @v_defdefine;
		END  
		CLOSE defCur;
		DEALLOCATE defCur;

		-- REINSTATE REFERENCE CONSTRAINTS ON COLUMN
		DECLARE refCur CURSOR FOR SELECT refdefine FROM #vt_refdefine; 
		OPEN refCur;
		FETCH NEXT FROM refCur INTO @v_refdefine;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				EXECUTE (@v_refdefine);
				FETCH NEXT FROM refCur INTO @v_refdefine;
		END  
		CLOSE refCur;
		DEALLOCATE refCur;

		-- REINSTATE INDEXES ON COLUMN
		DECLARE idxCur CURSOR FOR SELECT idxdefine FROM #vt_idxdefine; 
		OPEN idxCur;
		FETCH NEXT FROM idxCur INTO @v_idxdefine;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				PRINT (@v_idxdefine);
				EXECUTE (@v_idxdefine);
				FETCH NEXT FROM idxCur INTO @v_idxdefine;
		END  
		CLOSE idxCur;
		DEALLOCATE idxCur;
		
		RETURN @v_sqlstate;
END
GO


CREATE PROCEDURE $(CURRENT_SCHEMA).PROCESS_UOW(@MODULE_ID INT,@SUB_MODULE_ID INT,@UOW_ID INT,@ODS_SCHEMA VARCHAR(20),@DM_SCHEMA VARCHAR(20), @TARGET_VERSION VARCHAR(10),@REP_USER VARCHAR(20))
AS     
BEGIN
        

			DECLARE @MESSAGE_TEXT VARCHAR(100);
			DECLARE @RETVAL INT;
SET @RETVAL= 0; 
			DECLARE @v_sqlstate CHAR(5) ;
SET @v_sqlstate= '00000';
    		DECLARE @v_sqlcode INT ;
SET @v_sqlcode=0;
    		DECLARE @v_errmsg VARCHAR(100);
 SET @v_errmsg= '';
   		DECLARE @err_stmt VARCHAR(1000);
 SET @err_stmt='';
    	
   		DECLARE @v_module_id 							INT;
			DECLARE @v_sub_module_id 					INT;
      		DECLARE @v_uow_id 									INT;
      		DECLARE @v_ods_schema VARCHAR(20); 
	DECLARE @v_dm_schema VARCHAR(20);
			
	DECLARE @stmt VARCHAR(1000) ;
SET @stmt= ''; 
			DECLARE @v_statement_number 				INT;
			DECLARE @v_area 										VARCHAR(255);
			DECLARE @v_type 										VARCHAR(255);
			DECLARE @v_action 									VARCHAR(255);
			DECLARE @v_data_movement 					INT;
			DECLARE @v_table_refactoring 			INT;
			DECLARE @v_description 						VARCHAR(1000);
			DECLARE @v_affected_table 					VARCHAR(255);
			DECLARE @v_affected_schema 				VARCHAR(255);
			DECLARE @v_object_type 						VARCHAR(255);
			DECLARE @v_object_name 						VARCHAR(255);
			DECLARE @v_pre_exists 							INT;
			DECLARE @v_recoverable 						INT;
			DECLARE @v_has_dependencies 				INT;
			DECLARE @v_execution_status 				INT;
			DECLARE @v_execute 								INT;
			DECLARE @v_release 								VARCHAR(255);
			DECLARE @v_special_release 				VARCHAR(255);
			DECLARE @v_text 										VARCHAR(8000);
			DECLARE @v_temp_text 							VARCHAR(8000);
			DECLARE @v_failled_pred integer ;
SET @v_failled_pred= 0;
			DECLARE @v_target_version VARCHAR(255);
			DECLARE @v_rep_user VARCHAR(20);

SET @v_module_id=@MODULE_ID;
			SET @v_sub_module_id=@SUB_MODULE_ID;
			SET @v_uow_id = @UOW_ID;
			SET @v_ods_schema= @ODS_SCHEMA;
			SET @v_dm_schema = @DM_SCHEMA;
			SET @v_target_version= @TARGET_VERSION;
			SET @v_rep_user=@REP_USER;
			
		
			DECLARE C4 CURSOR FOR 
				SELECT STATEMENT_NUMBER,ACTION,AREA,DATA_MOVEMENT,TABLE_REFACTORING,AFFECTED_TABLE,
				AFFECTED_SCHEMA,OBJECT_TYPE,OBJECT_NAME,RECOVERABLE,HAS_DEPENDENCIES,TEXT
				FROM CONFIG.STATEMENT_LIST 
				WHERE MODULE_ID=@v_module_id AND SUB_MODULE_ID=@v_sub_module_id AND UOW_ID=@v_uow_id
				AND [EXECUTE]=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=@v_target_version AND PRE_EXISTS=0
				ORDER BY STATEMENT_NUMBER,UOW_ID;

				OPEN C4;
				FETCH NEXT FROM C4 INTO @v_statement_number,@v_action,@v_area,@v_data_movement,@v_table_refactoring,@v_affected_table,@v_affected_schema,@v_object_type,@v_object_name,@v_recoverable,@v_has_dependencies,@v_text;
				
				WHILE @@FETCH_STATUS = 0 
				BEGIN
				BEGIN TRANSACTION T1
				  IF (@v_has_dependencies=1) 
				  BEGIN
				  SELECT @v_failled_pred=COUNT(*) FROM CONFIG.STATEMENT_DEPENDENCIES WHERE STATEMENT_NUMBER=@v_statement_number AND EXISTS(SELECT STATEMENT_NUMBER FROM CONFIG.STATEMENT_LIST WHERE 
				  STATEMENT_NUMBER=PREDECESSOR_NUMBER AND (EXECUTION_STATUS <>1 OR EXECUTION_STATUS IS NULL)AND PRE_EXISTS<>1);
					  IF(@v_failled_pred>0)
						
						 GOTO Err70011;
					   
				  END
				  
				SELECT @v_temp_text=$(CURRENT_SCHEMA).REPLACE_SCHEMA(@v_text, @v_ods_schema,@v_dm_schema,@v_rep_user);				
				IF ((@v_action = 'DROP COLUMN') AND (CHARINDEX('ALTER TABLE', @v_temp_text) > 0))
						EXEC $(CURRENT_SCHEMA).PROCESS_ALL_CONSTR @v_affected_schema,@v_affected_table,@v_object_name;

				IF ((@v_action = 'ALTER TABLE') AND (CHARINDEX('ADD CONSTRAINT', @v_temp_text) > 0) AND (CHARINDEX(' DEFAULT ', @v_temp_text) > 0))
						EXEC $(CURRENT_SCHEMA).PROCESS_IMPLICIT_CONSTR @v_affected_schema,@v_affected_table,@v_object_name,'D';
					
				IF ((@v_action = 'ALTER DEFAULT') AND (CHARINDEX('ADD DEFAULT', @v_temp_text) > 0))
						EXEC $(CURRENT_SCHEMA).PROCESS_IMPLICIT_CONSTR @v_affected_schema,@v_affected_table,@v_object_name,'D';

				IF ((@v_action = 'ALTER COLUMN') AND (CHARINDEX('ALTER TABLE', @v_temp_text) > 0))
						EXEC $(CURRENT_SCHEMA).PROCESS_COLUMN_UPDATE @v_affected_schema,@v_affected_table,@v_object_name,@v_temp_text;
				ELSE				
						EXECUTE (@v_temp_text);

				SELECT @v_sqlstate=@@ERROR;
				IF (@v_sqlstate<>0) 					 
					   GOTO Err70010;
				ELSE
				 
				  	UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=1, RET_CODE='00000' WHERE STATEMENT_NUMBER=@v_statement_number;
				    SELECT @v_sqlstate=@@ERROR;
					IF (@v_sqlstate<>0) 
					   GOTO Err70010;
					   
					COMMIT TRANSACTION T1;   
					FETCH NEXT FROM C4 INTO @v_statement_number,@v_action,@v_area,@v_data_movement,@v_table_refactoring,@v_affected_table,@v_affected_schema,@v_object_type,@v_object_name,@v_recoverable,@v_has_dependencies,@v_text;  
					--FETCH NEXT FROM C4;
				END;
				CLOSE C4;
				DEALLOCATE C4;
				
		 GOTO pcEnd
		 

Err70010:
ROLLBACK TRANSACTION T1; 
UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=0,RET_CODE=@v_sqlstate WHERE STATEMENT_NUMBER=@v_statement_number;
--COMMIT;
RAISERROR('$(CRREWMSGE101)',70010, 1) WITH LOG,NOWAIT
RETURN ;

Err70011:
ROLLBACK TRANSACTION T1; 
UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=0,RET_CODE=@v_sqlstate WHERE STATEMENT_NUMBER=@v_statement_number;
--COMMIT;
RAISERROR('$(CRREWMSGE102)',70011, 1) WITH LOG,NOWAIT
RETURN ;

pcEnd:
RETURN @v_sqlstate;
END
GO

CREATE PROCEDURE $(CURRENT_SCHEMA).VALIDATE_EXISTENCE(@STATEMENT_NUMBER INT, @TYPE VARCHAR(255),@ACTION VARCHAR(255),@AFFECTED_TABLE VARCHAR(255),@AFFECTED_SCHEMA VARCHAR(255),@OBJECT_TYPE VARCHAR(255),@OBJECT_NAME VARCHAR(255),@ODS_SCHEMA VARCHAR(20),@DM_SCHEMA VARCHAR(20))
AS    
BEGIN


    	DECLARE @v_sqlcode INT ;
SET @v_sqlcode=0;
    	DECLARE @v_errmsg VARCHAR(100) 
SET @v_errmsg= '';
    	DECLARE @err_stmt VARCHAR(1000) 
SET @err_stmt= '';
    	DECLARE @v_exists INT 
SET  @v_exists=0;
    	DECLARE @v_target_version VARCHAR(10);
    	DECLARE @v_module INT;
			DECLARE @v_sub_module INT;
			DECLARE @v_ods_schema VARCHAR(20); 
			DECLARE @v_dm_schema VARCHAR(20);
			DECLARE @stmt VARCHAR(1000) 
SET @stmt=''; 
			DECLARE @v_statement_number 				INT;
			DECLARE @v_module_id 							INT;
			DECLARE @v_sub_module_id 					INT;
			DECLARE @v_area 										VARCHAR(255);
			DECLARE @v_type 										VARCHAR(255);
			DECLARE @v_action 									VARCHAR(255);
			DECLARE @v_uow_id 									INT;
			DECLARE @v_data_movement 					INT;
			DECLARE @v_table_refactoring 			INT;
			DECLARE @v_description 						VARCHAR(1000);
			DECLARE @v_affected_table 					VARCHAR(255);
			DECLARE @v_affected_schema 				VARCHAR(255);
			DECLARE @v_object_type 						VARCHAR(255);
			DECLARE @v_object_name 						VARCHAR(255);
			DECLARE @v_pre_exists 							INT;
			DECLARE @v_recoverable 						INT;
			DECLARE @v_has_dependencies 				INT;
			DECLARE @v_execution_status 				INT;
			DECLARE @v_execute 								INT;
			DECLARE @v_release 								VARCHAR(255);
			DECLARE @v_special_release 				VARCHAR(255);
    	
    	SET @v_action=@ACTION;
    	
    	SET @v_affected_schema= CASE WHEN @AFFECTED_SCHEMA='RIODS' THEN @ODS_SCHEMA ELSE @DM_SCHEMA END;
    	SET @v_affected_table=@AFFECTED_TABLE;
    	SET @v_object_name=@OBJECT_NAME;
    	SET @v_type=@TYPE;    	
    	SET @v_statement_number=@STATEMENT_NUMBER;
    	
  	IF @v_action='ADD COLUMN'
       BEGIN
       	SELECT @v_exists=COUNT(*) FROM (SELECT  S.NAME AS TABLE_SCHEMA,T.NAME AS TABLE_NAME, C.NAME AS COLUMN_NAME,   
        UPPER(TP.NAME) AS DATA_TYPE  
		FROM sys.tables AS T  
		   JOIN  sys.schemas S ON S.SCHEMA_ID=T.SCHEMA_ID
		   JOIN sys.columns C ON T.OBJECT_ID = C.OBJECT_ID  
		  JOIN sys.types TP ON C.SYSTEM_TYPE_ID = TP.SYSTEM_TYPE_ID AND C.USER_TYPE_ID = TP.USER_TYPE_ID   
		WHERE T.IS_MS_SHIPPED = 0 ) C
		WHERE C.TABLE_NAME= @v_affected_table AND C.COLUMN_NAME=@v_object_name AND C.DATA_TYPE=@v_type;
      	IF @v_exists=0
            
       		SELECT  @v_exists= COUNT(*) FROM (SELECT  S.NAME AS TABLE_SCHEMA,T.NAME AS TABLE_NAME, C.NAME AS COLUMN_NAME,   
            UPPER(TP.NAME) AS DATA_TYPE  
			FROM sys.tables AS T  
			   JOIN  sys.schemas S ON S.SCHEMA_ID=T.SCHEMA_ID
			   JOIN sys.columns C ON T.OBJECT_ID = C.OBJECT_ID  
			  JOIN sys.types TP ON C.SYSTEM_TYPE_ID = TP.SYSTEM_TYPE_ID AND C.USER_TYPE_ID = TP.USER_TYPE_ID   
			WHERE T.IS_MS_SHIPPED = 0 ) C
			WHERE C.TABLE_NAME= @v_affected_table AND C.COLUMN_NAME=@v_object_name;
       	ELSE
       		SET @v_exists=-1;	
  END;

      IF @v_action='CREATE CONSTRAINT'
      BEGIN
        IF @v_type='PRIMARY' 
      			SELECT @v_exists=COUNT(*) FROM (SELECT O.NAME AS PKNAME,S.NAME AS SCHEMA_NAME
				FROM  sys.all_objects O
				 JOIN sys.schemas S ON S.SCHEMA_ID=O.SCHEMA_ID
				WHERE O.TYPE='PK') C
				WHERE C.PKNAME=@v_object_name AND C.SCHEMA_NAME=@v_affected_schema;
      			ELSE
      			SELECT @v_exists=COUNT(*) FROM (SELECT O.NAME AS FKNAME,S.NAME AS SCHEMA_NAME
				FROM  sys.all_objects O
				 JOIN sys.schemas S ON S.SCHEMA_ID=O.SCHEMA_ID
				WHERE O.TYPE='F') C
				WHERE C.FKNAME=@v_object_name AND C.SCHEMA_NAME=@v_affected_schema;
      END;
	
    
      IF @v_action= 'CREATE INDEX' 
      		SELECT @v_exists=COUNT(*) FROM (SELECT NAME FROM sys.indexes ) C WHERE  C.NAME= @v_object_name ;
      IF @v_action= 'CREATE TABLE' 
             SELECT @v_exists=COUNT(*)  FROM (SELECT  S.NAME AS TABLE_SCHEMA,T.NAME AS TABLE_NAME
			FROM sys.tables AS T  
			   JOIN  sys.schemas S ON S.SCHEMA_ID=T.SCHEMA_ID) C
			  WHERE C.TABLE_SCHEMA=@v_affected_schema AND C.TABLE_NAME=@v_object_name;
      IF @v_action= 'CREATE TRIGGER' 
      	  SELECT @v_exists=COUNT(*) FROM (SELECT O.NAME AS TRIG_NAME,S.NAME AS SCHEMA_NAME
				FROM  sys.all_objects O
				 JOIN sys.schemas S ON S.SCHEMA_ID=O.SCHEMA_ID
				WHERE O.TYPE='TR') C WHERE C.SCHEMA_NAME= @v_affected_schema AND C.TRIG_NAME = @v_object_name;
      IF @v_action= 'CREATE VIEW' 
      	  SELECT @v_exists=COUNT(*) FROM (SELECT V.NAME AS VIEW_NAME,S.NAME AS SCHEMA_NAME
				FROM  sys.views V
				 JOIN sys.schemas S ON S.SCHEMA_ID=V.SCHEMA_ID) C  WHERE C.SCHEMA_NAME= @v_affected_schema AND C.VIEW_NAME = @v_object_name;

      
      UPDATE CONFIG.STATEMENT_LIST SET PRE_EXISTS=CASE WHEN @v_exists>0 THEN 1 WHEN @v_exists=-1 THEN 2 ELSE 0 END WHERE STATEMENT_NUMBER=@v_statement_number;
--      COMMIT;
      UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=CASE WHEN PRE_EXISTS>0 THEN 2 ELSE EXECUTION_STATUS END WHERE STATEMENT_NUMBER=@v_statement_number;
--      COMMIT;  
END
GO


CREATE PROCEDURE $(CURRENT_SCHEMA).MIGRATE_MODULE(@MODULE_ID INT,@SUB_MODULE_ID INT,@ODS_SCHEMA VARCHAR(20),@DM_SCHEMA VARCHAR(20),@TARGET_VERSION VARCHAR(10),@REP_USER VARCHAR(20))
AS
BEGIN

      
		DECLARE @v_sqlcode INT ;
		SET @v_sqlcode = 0;
    	DECLARE @v_errmsg VARCHAR(100);
    	DECLARE @err_stmt VARCHAR(1000);
    	
    	DECLARE @v_target_version VARCHAR(10);
    	DECLARE @v_module INT;
			DECLARE @v_sub_module INT;
			DECLARE @v_ods_schema VARCHAR(20); 
			DECLARE @v_dm_schema VARCHAR(20);
			DECLARE @stmt VARCHAR(1000) ; 
			DECLARE @v_statement_number 				INT;
			DECLARE @v_module_id 							INT;
			DECLARE @v_sub_module_id 					INT;
			DECLARE @v_area 										VARCHAR(255);
			DECLARE @v_type 										VARCHAR(255);
			DECLARE @v_action 									VARCHAR(255);
			DECLARE @v_uow_id 									INT;
			DECLARE @v_data_movement 					INT;
			DECLARE @v_table_refactoring 			INT;
			DECLARE @v_description 						VARCHAR(1000);
			DECLARE @v_affected_table 					VARCHAR(255);
			DECLARE @v_affected_schema 				VARCHAR(255);
			DECLARE @v_object_type 						VARCHAR(255);
			DECLARE @v_object_name 						VARCHAR(255);
			DECLARE @v_pre_exists 							INT;
			DECLARE @v_recoverable 						INT;
			DECLARE @v_has_dependencies 				INT;
			DECLARE @v_execution_status 				INT;
			DECLARE @v_execute 								INT;
			DECLARE @v_release 								VARCHAR(255);
			DECLARE @v_special_release 				VARCHAR(255);
			DECLARE @v_text 										VARCHAR(8000);
			DECLARE @sp_stmt                   VARCHAR(1000);
			DECLARE @fetch_code CHAR(5);
			DECLARE @v_rep_user VARCHAR(20);
			
			SET @v_module=@MODULE_ID;
			SET @v_sub_module=@SUB_MODULE_ID;
			SET @v_target_version=@TARGET_VERSION;
			SET @v_ods_schema=@ODS_SCHEMA;
			SET @v_dm_schema=@DM_SCHEMA;
			SET @v_rep_user=@REP_USER;
						
			DECLARE C2 CURSOR FOR 
				SELECT STATEMENT_NUMBER,MODULE_ID,SUB_MODULE_ID,AREA,TYPE,ACTION,UOW_ID,DATA_MOVEMENT,TABLE_REFACTORING,AFFECTED_TABLE,
				AFFECTED_SCHEMA,OBJECT_TYPE,OBJECT_NAME,RECOVERABLE,HAS_DEPENDENCIES,EXECUTION_STATUS,[EXECUTE],RELEASE,SPECIAL_RELEASE,TEXT
				FROM CONFIG.STATEMENT_LIST 
				WHERE MODULE_ID=@v_module AND SUB_MODULE_ID=@v_sub_module 
				AND [EXECUTE]=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=@v_target_version
				ORDER BY STATEMENT_NUMBER,UOW_ID;
				OPEN C2;
				FETCH NEXT FROM C2 INTO @v_statement_number,	@v_module_id,@v_sub_module_id,@v_area,@v_type,@v_action,@v_uow_id,@v_data_movement,@v_table_refactoring,@v_affected_table,@v_affected_schema,@v_object_type,@v_object_name,@v_recoverable,@v_has_dependencies,@v_execution_status,@v_execute,@v_release,@v_special_release,@v_text;
				WHILE @@FETCH_STATUS = 0 
				BEGIN
					EXEC $(CURRENT_SCHEMA).VALIDATE_EXISTENCE @v_statement_number,@v_type,@v_action,@v_affected_table,@v_affected_schema,@v_object_type,@v_object_name,@v_ods_schema,@v_dm_schema;
						  
	                FETCH NEXT FROM C2 INTO @v_statement_number,	@v_module_id,@v_sub_module_id,@v_area,@v_type,@v_action,@v_uow_id,@v_data_movement,@v_table_refactoring,@v_affected_table,@v_affected_schema,@v_object_type,@v_object_name,@v_recoverable,@v_has_dependencies,@v_execution_status,@v_execute,@v_release,@v_special_release,@v_text;
	                --FETCH NEXT FROM C2; 
				END 
				CLOSE C2;
				DEALLOCATE C2;

			

			DECLARE C3 CURSOR FOR 
			SELECT DISTINCT MODULE_ID,SUB_MODULE_ID,UOW_ID
				FROM CONFIG.STATEMENT_LIST 
				WHERE MODULE_ID=@v_module AND SUB_MODULE_ID=@v_sub_module 
				AND [EXECUTE]=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=@v_target_version AND PRE_EXISTS=0
				GROUP BY MODULE_ID,SUB_MODULE_ID,UOW_ID
				ORDER BY UOW_ID,MODULE_ID,SUB_MODULE_ID;
				OPEN C3;
				FETCH NEXT FROM C3 INTO @v_module_id,@v_sub_module_id,@v_uow_id;
				WHILE @@FETCH_STATUS = 0 
				BEGIN
						EXEC $(CURRENT_SCHEMA).PROCESS_UOW @v_module_id,@v_sub_module_id,@v_uow_id,@v_ods_schema,@v_dm_schema,@v_target_version,@v_rep_user;
						SET @v_sqlcode=@@ERROR;
						FETCH NEXT FROM C3 INTO @v_module_id,@v_sub_module_id,@v_uow_id;
						--FETCH NEXT FROM C3;
    			 END
    			CLOSE C3;
    			DEALLOCATE C3;

END
GO

CREATE PROCEDURE $(CURRENT_SCHEMA).DW_MIGRATE(@ODS_SCHEMA VARCHAR(20),@DM_SCHEMA VARCHAR(20),@REP_USER VARCHAR(20))
AS    
BEGIN
     
		DECLARE @v_sqlcode INT; 
         SET @v_sqlcode=0;
    	DECLARE @err_stmt VARCHAR(1000);
    	
    	
			DECLARE @v_target_version VARCHAR(10) ;
			SET  @v_target_version='1.0.0.1';
			DECLARE @v_current_version VARCHAR(10);
			DECLARE @v_migration_version VARCHAR(10);
			DECLARE @v_module INT;
			DECLARE @v_sub_module INT;
			DECLARE @v_ods_schema VARCHAR(20); 
			DECLARE @v_dm_schema VARCHAR(20);
			DECLARE @stmt VARCHAR(1000); 
			DECLARE @sp_stmt VARCHAR(1000);
			DECLARE @v_rep_user VARCHAR(20);

			
			
			SET @v_ods_schema=@ODS_SCHEMA;
			SET @v_dm_schema=@DM_SCHEMA;
			SET @v_rep_user=@REP_USER;
			
			SELECT @v_current_version=CHAR_VALUE FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=6;
			
			SELECT @v_target_version=CHAR_VALUE FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=7;
			
			IF (@v_current_version IS NULL) 
						RAISERROR('$(CRREWMSGE098)',70001, 1) WITH LOG,NOWAIT

			IF (@v_target_version IS NULL)
						RAISERROR('$(CRREWMSGE099)',70002, 1) WITH LOG,NOWAIT;
			
      IF (@v_current_version > @v_target_version) 
						RAISERROR('$(CRREWMSGE100)',70005, 1) WITH LOG,NOWAIT;
			
			IF (@v_ods_schema IS NULL)
						RAISERROR('$(CRREWMSGE005)',70003, 1) WITH LOG,NOWAIT;
			
			
			IF (@v_dm_schema IS NULL)
						RAISERROR('$(CRREWMSGE006)',70004, 1) WITH LOG,NOWAIT;
			
			IF (@v_rep_user IS NULL)
						RAISERROR('$(CRREWMSGE043)',70006, 1) WITH LOG,NOWAIT;


		DECLARE CVer CURSOR FOR 		-- the versions to need to be migrated 
				SELECT DISTINCT S.RELEASE,S.MODULE_ID, S.SUB_MODULE_ID FROM CONFIG.STATEMENT_LIST S WHERE S.RELEASE >= @v_current_version and S.RELEASE <= @v_target_version 
				AND EXISTS(SELECT M.MODULE_ID, M.SUB_MODULE_ID FROM CONFIG.MODULES M WHERE M.MODIFY='Y' AND M.MODULE_ID=S.MODULE_ID AND M.SUB_MODULE_ID=S.SUB_MODULE_ID)
						ORDER BY S.RELEASE, S.MODULE_ID, S.SUB_MODULE_ID;
		      

		OPEN CVer;
		FETCH NEXT FROM CVer INTO @v_migration_version,@v_module,@v_sub_module;
		WHILE @@FETCH_STATUS = 0
		BEGIN
				EXEC $(CURRENT_SCHEMA).MIGRATE_MODULE @v_module,@v_sub_module,@v_ods_schema,@v_dm_schema,@v_migration_version,@v_rep_user;
				FETCH NEXT FROM CVer INTO @v_migration_version,@v_module,@v_sub_module;
		END 
		CLOSE CVer;
		DEALLOCATE CVer;
 			
		UPDATE CONFIG.SYSCONFIG_INFO SET CHAR_VALUE=@v_target_version WHERE INFO_ID=6;
		--DELETE FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=7;

END
GO

